# -*- coding: utf-8 -*-

"""
Goal here is to make a script that can take in the spell attributes
on the command line and output a png as required. Also useful if it can
return a spell class

"""
from SpellWriting.scripts.spells import spell
import argparse
import line_shapes
import SpellWriting.scripts.bases


def create_parser_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("-lvl","--level",help = "necessary input: level of the spell",
                        required = True)
    parser.add_argument("-sch","--school",help = "necessary input: school of the spell",
                        required = True)
    parser.add_argument("-dmg","--damagetype",help = "necessary input: damage type of spell",
                        required = True)
    parser.add_argument("-a","--aoe",help = "necessary input: AoE of the spell, format is `<shape> (<size>)` e.g. `cube (30)` CHECK THE TXT FILES IF YOU CAN'T FIND WHAT YOU'RE LOOKING FOR. ADD TO THE FILES IF WHAT YOU WANT ISN'T THERE (e.g. a new damage type, or a mix of the two)",
                        required = True)
    parser.add_argument("-ran","--range",help = "necessary input: range of the spell, format is `<size> <units>` e.g. `30 feet`. Also inlcudes one word versions like `Self`.",
                        required = True)
    parser.add_argument("-dur","--duration",help = "necessary input: duration of the spell, format is `<n> <units>` e.g. `1 hour`. Also inlcludes the `Up to...`",
                        required = True)
    
    #concentration and ritual
    parser.add_argument("-r","--ritual",help = "include if spell is ritual",action = "store_true")
    parser.add_argument("-c","--concentration",help = "include if spell is ritual",action = "store_true")

    #plot variables
    parser.add_argument("-spell_name",help = "Spell Name for Title",default = "Test")
    parser.add_argument("--savename",help = "Savename of plot",default = "output.png")
    parser.add_argument("--annotate",help = "Whether to annotate the spell dir",action = "store_true")
    parser.add_argument("--show_all_paths",help = "Whether to show all possible paths",action = "store_true")
    parser.add_argument("--save_dpi",help = "DPI to save image at",default = 200)
    parser.add_argument("--cmap",help = "Colour map to use for annotating",default = "magma")
    parser.add_argument("--dot_color",help = "Colour of the dots in the drawing",default = 'k')
    parser.add_argument("--line_color",help = "Colour of the lines (if not annotating)",default = 'darkred')
    
    return parser
    
if __name__=="__main__":
    
    parser = create_parser_args()
    args = parser.parse_args()
    spell_obj = spell(args)
    spell_obj.draw(args.annotate,
                   show_all_paths=args.show_all_paths,
                   savename = args.savename,output_dpi=int(args.save_dpi),
                   line_color = args.line_color,
                   dot_color = args.dot_color,
                   cmap = args.cmap,
                   show_name = True)


    



# -*- coding: utf-8 -*-
"""
Created on Sat Mar 14 16:07:36 2026

@author: trent
"""

import typer

app = typer.Typer(no_args_is_help=True)

from glyphloom.cli.commands import (fe)

app.add_typer(fe.app, name='5e')


# -*- coding: utf-8 -*-
"""
Created on Sun Mar 15 14:23:40 2026

@author: trent
"""
import typing

if __name__ != "__main__":
    import matplotlib
    matplotlib.use("TkAgg")
import matplotlib.pyplot as plt
plt.rcParams.update({
    "figure.facecolor":  (0.0, 0.0, 0.0, 0.0),
    "axes.facecolor":    (1.0, 1.0, 1.0, 0.0),
    "savefig.facecolor": (0.0, 0.0, 0.0, 0.0),
})
from numpy import linspace

from glyphloom.generation import data, geometry


class Glyph:
    
    def __init__(self,
                 spelldata: data.SpellData = None,
                 leylines: geometry.Leylines = None) -> typing.Self:
    
        self.spelldata = spelldata
        if leylines:
            self.leylines = leylines
        else:
            self.leylines = geometry.Leylines(
                geometry.Founts(n=len(spelldata.collect_attributes())))
        
    
    def draw(self, legend: bool = False, legend_kwargs: dict = {}):
        # TODO: Add thematic colors
        fig = plt.figure(constrained_layout=True)
        ax = fig.add_subplot(111)
        ax.set_aspect('equal', adjustable='box')
        plt.title(self.spelldata.name)
        
        if not self.leylines.founts.n_points:
            fig.show()
            return fig, ax
        
        attrs = self.spelldata.collect_attributes().values()
        cmap = plt.get_cmap('cubehelix')
        colors = cmap(linspace(0.1, 0.8, len(attrs)))
        non_glyph = 0
        for attr in attrs:
            value = getattr(self.spelldata, attr.name)
            if not attr.glyph:
                if value and non_glyph:
                    kwargs = dict(fillstyle='none', 
                                  markersize=20+10*non_glyph,
                                  color='#991B1E')
                    plt.plot(0,0, 'o', **kwargs)
                elif value:
                    kwargs = dict(markersize=15, color='#991B1E')
                    plt.plot(0,0, 'o', **kwargs)
                
                non_glyph+=1
                continue
            order = attr.order
            options = attr.options
            
            if value is None: continue
            index = list(options).index(value)
            binary = self.leylines.necklace[index]
            paths = self.leylines.default_curves[order, binary.astype(bool)]
            for i, path in enumerate(paths):
                if i:
                    plt.plot(*path, color=colors[order])
                else:
                    plt.plot(*path, color=colors[order], label=attr.name)
        plt.plot(*self.leylines.founts[:,1:], 'ok', fillstyle='full', markersize=8)
        plt.plot(*self.leylines.founts[:,0], 'ok', fillstyle='none', markersize=8)
        if legend and legend_kwargs:
            plt.legend(**legend_kwargs)
        elif legend:
            ax.legend(loc='upper left',
                      bbox_to_anchor=(1.02, 0.8))

        plt.axis('off')
        
        return fig, ax
            
if __name__ == '__main__':
    from glyphloom.data.fifth_edition import SpellData_5e
    
    spelldata = SpellData_5e.get_spell('Wish', 
                                       'offline')
    leylines = geometry.Leylines(
        founts=geometry.Founts(
            n_points=13,
            expression=None),
        expression='linear')
    Glyph(spelldata, leylines).draw(legend=True)